'''
the import user command looks for .pythonrc.py and execute it
in our case .pythonrc.py gives acces to the modules from
$HOME/control/python/pythonLibs
'''
import user

import numarray as na
import struct
import time
import itertools

### modules in pythonLibs
from socketLib import initSocket
import camStructures as camStructures


class IRCamCommands:
    def __init__(self, host = '192.168.3.136', port = 60000):
        """ constructor, setting initial variables """
        self.host = host
        self.port = port
        self.champhost = '192.168.3.15'
	self.champhost = '193.168.3.14'
	self.champport = 6001
        self.champhost = '192.168.3.14'
	self.champport = 6001
        self.gdtInfo = na.zeros(7, type=na.UInt32)
        self.delayLinesOffsets = na.zeros(6, type=na.Float64)
        self.updatedOffsets = na.zeros(6, type=na.Float64)
        self.weights = na.zeros(6, type=na.Float32)
        self.opds  = na.zeros(6, type=na.Float32)

        self.daq = camStructures.theDaq
        self.daqKeys = camStructures.theDaqKeys
        self.daqValues = camStructures.theDaqValues
        self.daqFormat = camStructures.theDaqFormat
        self.daqType = camStructures.theDaqType
        #print "self.daq = ", self.daq
        
        self.pythonConfig = camStructures.pythonConfig
        self.pythonConfigKeys = camStructures.pythonConfigKeys
        self.pythonConfigValues = camStructures.pythonConfigValues
        self.pythonConfigFormat = camStructures.pythonConfigFormat
        print "self.pythonConfig = ", self.pythonConfig

        self.useBackground = False
        self.numberOfBackgroundFrames = 1000

	self.CHAMPOffsets = na.zeros(6, type=na.Float64)

    def sendCommand(self, command):
        self.sock = initSocket(self.host, self.port)
        self.sock.send(command)
        data = self.sock.recv(2)
        print 'Received', repr(data)

    def sendStructCommand(self, command, c_struct):
        print self.host,self.port
        self.sock = initSocket(self.host, self.port)
        if self.sock == None :
            print "socket failed to open"
            return -1
        else:
            strLen = len(c_struct)
            self.sock.send(command)
            print "sent %s" %(command)
            self.sock.send(struct.pack('L',strLen))
            self.sock.send(c_struct)
            data = self.sock.recv(2)
            self.sock.close()
            print 'Received', repr(data)

    def getStructFromRT(self, command, c_struct):
        print self.host
        print self.port
        self.sock = initSocket(self.host, self.port)
        strLen = len(c_struct)
        if self.sock != None:
            self.sock.send(command)
            print "sent %s" %(command)
            self.sock.send(struct.pack('L',strLen))
            self.sock.send(c_struct)
            l = 0
            data = ''
            while l < strLen:
                message = self.sock.recv(strLen-l)
                l += len(message)
                data += message
            self.sock.close()
            print 'Received'
            return data
        else: return None
    
    def startExposure(self):
        self.sendCommand('SX')

    def stopExposure(self):
        self.sendCommand('XX')

    def abortExposure(self):
        print 'abort button disabled'

    def getBackgroundFrame(self):
        self.sendCommand('TB')

    def resetBackgroundFrame(self):
        self.sendCommand('RB')

    def useBackgroundFrame(self):
        self.sendCommand('UB')

    def resetCont(self):
        self.sendCommand('IO')

    def setBias(self):
        self.sendCommand('SB')

    def setCont(self):
        self.sendCommand('SC')

    def sendToOple(self):
        self.sendCommand('OP')

    def updateDaqStructure(self):
        values = [self.daq[key] for key in self.daqKeys]
        cstruct = struct.pack('8HL6B4f30s20sB2H', *values)
        self.sendStructCommand('CN', cstruct)

    def updateDoImageSave(self):
        values = [self.daq[key] for key in self.daqKeys]
        cstruct = struct.pack('8HL6B4f30s20sB2H', *values)
        self.sendStructCommand('SD', cstruct)

    def sendStruct(self):
        values = [self.daq[key] for key in self.daqKeys]
        cstruct = struct.pack('8HL6B4f30s20sB2H', *values)
        self.sendStructCommand('AQ', cstruct)

    def getStruct(self):
        values = [self.daq[key] for key in self.daqKeys]
        cstruct = camStructures.packTheDaq(values)
        data = self.getStructFromRT('RC', cstruct)
        if data == None:
            print "Didn't receive daq from RT"
        else:
            self.daq = camStructures.unpackTheDaq(data)
            self.daqValues = [self.daq[key] for key in self.daqKeys]

    def sendPythonConfigStruct(self):
        values = [self.pythonConfig[key] for key in self.pythonConfigKeys]
        print "self.pythonConfig['fileDescription'] = ",self.pythonConfig['fileDescription']
        cstruct = camStructures.packPythonConfig(values)
        self.sendStructCommand('RP', cstruct)

    def getPythonConfigStruct(self):
        values = [self.pythonConfig[key] for key in self.pythonConfigKeys]
        cstruct = camStructures.packPythonConfig(values)
        data = self.getStructFromRT('SP',cstruct)
        if data == None:
            print "Didn't receive pythonConfig from RT"
        else:
            self.pythonConfig = camStructures.unpackPythonConfig(data)
            self.pythonConfigValues = [self.pythonConfig[key] for key in self.pythonConfigKeys]

    def killServer(self):
        self.sock = initSocket(self.host, self.port)
        self.sock.send('EN')
        self.sock.send(struct.pack('L',1))
        self.sock.send(struct.pack('H',0))
        data = self.sock.recv(2)
        self.sock.close()
        print 'Received', repr(data)

    def getDelayLinesOffsets(self):
        self.sock = initSocket(self.host, self.port)
        self.sock.settimeout(5)
        strLen = 48
        self.sock.send('OF')
        self.sock.send(struct.pack('L',strLen))
        data = self.sock.recv(strLen)
        self.sock.close()
        self.delayLinesOffsets = na.fromstring(data, na.Float64, 6)

    def getFringeInfo(self):
        self.sock = initSocket(self.host, self.port)
        self.sock.settimeout(5)
        strLen = 48
        self.sock.send('FI')
        self.sock.send(struct.pack('L',strLen))
        data = self.sock.recv(strLen)
        self.sock.close()
        self.opds = na.fromstring(data, na.Float32, (2, 6))[0]
        self.weights = na.fromstring(data, na.Float32, (2, 6))[1]

    def sendGdtInfo(self):
        self.sock = initSocket(self.host, self.port)
        self.sock.settimeout(5)
        gdtStruct = struct.pack('7I',*self.gdtInfo)
        strLen = len(gdtStruct)
        self.sock.send('GI')
        self.sock.send(struct.pack('L',strLen))
        self.sock.send(gdtStruct)
        data = self.sock.recv(2)
        self.sock.close()

    def sendGdtOff(self):
        self.sock = initSocket(self.host, self.port)
	self.sock.settimeout(5)
	gdtStruct = struct.pack('7I',0,0,0,0,0,0,0)
	strLen = len(gdtStruct)
	self.sock.send('GI')
	self.sock.send(struct.pack('L',strLen))
	self.sock.send(gdtStruct)
	data = self.sock.recv(2)
	self.sock.close()

    def sendCHAMPtest(self):
        print "opening host/socket: ", self.champhost,self.champport 
	self.sock = initSocket(self.champhost, self.champport)
	if self.sock == None :
	  print "socket failed to open"
	  return -1
	else:
	  self.sock.settimeout(5)
	  temp = self.CHAMPOffsets
	  temp =temp[::-1] 
	  opdStruct = struct.pack('6d', *temp)
	  # reverse string for little endian
	  opdStruct = opdStruct[::-1]
	  strLen = len(opdStruct)
	  print "Sending CHAMP offsets: ",self.CHAMPOffsets
	  temp=self.CHAMPOffsets
	  print temp[::-1]
	  print "opdstruct: ",opdStruct
	  #  self.sock.send('OP')
	  # self.sock.send(struct.pack('L',strLen))
	  self.sock.send(opdStruct)
	  data = self.sock.recv(2)

	  self.sock.send(opdStruct)
	  data = self.sock.recv(2)

	  self.sock.send(opdStruct)
	  data = self.sock.recv(2)

	  self.sock.send(opdStruct)
	  data = self.sock.recv(2)

	  # time.sleep(20)
	  self.CHAMPOffsets = [-1000.,-1000.,-1000.,-1000.,-1000.,-1000.]
	  temp = self.CHAMPOffsets
	  temp =temp[::-1]
	  opdStruct = struct.pack('6d', *temp)
	  opdStruct = opdStruct[::-1]

	  print "Sending kill sequence: ",self.CHAMPOffsets
	  self.sock.send(opdStruct)
	  data = self.sock.recv(2)
	  print "Response: ",data
	  print "Waiting 5 seconds, then closing socket"
	  time.sleep(5)
	  self.sock.close()

    def sendUpdatedOffsets(self):
        self.sock = initSocket(self.host, self.port)
        self.sock.settimeout(5)
        opdStruct = struct.pack('6d', *self.updatedOffsets)
        strLen = len(opdStruct)
        self.sock.send('OP')
        self.sock.send(struct.pack('L',strLen))
        self.sock.send(opdStruct)
        data = self.sock.recv(2)
        self.sock.close()

    def coaddCount(self):
        self.sock = initSocket(self.host, self.port)
        self.sock.send('CC')
        print "sent CC"
        data = self.sock.recv(4)
        self.sock.close()
        ##print 'Received', data
        print 'Received', na.fromstring(data, na.Int32)[0]
        return na.fromstring(data, na.Int32)[0]

    def configureDetector(self):
        self.sendStruct()
        time.sleep(.5)
        self.resetCont()
        time.sleep(.5)
        self.setCont()
        time.sleep(.5)
        self.setBias()		
        time.sleep(.5)
